package dev.fitko.fitconnect.client.util;

import dev.fitko.fitconnect.api.exceptions.client.FitConnectInitialisationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.yaml.snakeyaml.Yaml;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

public final class RessourceLoadingUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(RessourceLoadingUtils.class);

    public static <T> T loadYaml(String path, Class<T> returnType) {
        try (final InputStream is = RessourceLoadingUtils.class.getClassLoader().getResourceAsStream(path)) {
            return new Yaml().loadAs(is, returnType);
        } catch (final Exception e) {
            throw new FitConnectInitialisationException("Yaml resource from path  " + path + " could not be loaded", e);
        }
    }

    public static String readSchemaResourceFromPath(String schemaPath) {
        return new String(readResourceToByteArray(schemaPath));
    }

    public static List<String> readSchemaResourcesFromPaths(List<String> schemaPaths) {
        return schemaPaths.stream()
                .map(RessourceLoadingUtils::readResourceToByteArray)
                .map(String::new)
                .collect(Collectors.toList());
    }

    public static String readKeyFromPath(final String keyPath) {
        try {
            return Files.readString(Path.of(keyPath));
        } catch (final IOException | NullPointerException e) {
            LOGGER.error("Could not read key from path '{}'. Please provide a valid path in your config.yml", keyPath);
            throw new FitConnectInitialisationException(e.getMessage(), e);
        }
    }

    public static byte[] readResourceToByteArray(final String resourcePath) {
        try (final InputStream in = RessourceLoadingUtils.class.getResourceAsStream(resourcePath)) {
            return Objects.requireNonNull(in).readAllBytes();
        } catch (final IOException | NullPointerException exception) {
            throw new FitConnectInitialisationException("Failed loading resource " + resourcePath, exception);
        }
    }

    public static String readResourceToString(final String resourcePath) {
        try (final InputStream in = RessourceLoadingUtils.class.getResourceAsStream(resourcePath)) {
            return new String(Objects.requireNonNull(in).readAllBytes(), StandardCharsets.UTF_8);
        } catch (final IOException | NullPointerException exception) {
            throw new FitConnectInitialisationException("Failed loading resource " + resourcePath, exception);
        }
    }

}
