package dev.fitko.fitconnect.client.util;

import com.nimbusds.jose.jwk.JWK;
import dev.fitko.fitconnect.api.domain.crypto.JWKPair;
import dev.fitko.fitconnect.jwkvalidator.JWKValidator;
import dev.fitko.fitconnect.jwkvalidator.exceptions.JWKValidationException;
import dev.fitko.fitconnect.jwkvalidator.exceptions.LogLevel;
import org.junit.jupiter.api.Test;

import java.util.UUID;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.is;
import static org.junit.jupiter.api.Assertions.assertTrue;

class ReplyChannelKeyGeneratorTest {


    @Test
    void generateKeyPairWithRandomIdPart() {

        // Given
        final JWKPair keyPair = ReplyChannelKeyGenerator.generateKeyPair();

        // When
        final JWK publicKey = keyPair.getPublicKey();
        final JWK privateKey = keyPair.getPrivateKey();

        // Then
        assertThat(publicKey.getKeyID(), containsString("reply-key"));
        assertTrue(publicKey.getKeyID().split(":")[1].matches("[a-f0-9]{8}(-[a-f0-9]{4}){4}[a-f0-9]{8}"));

        assertThat(privateKey.getKeyID(), containsString("reply-key"));
        assertTrue(privateKey.getKeyID().split(":")[1].matches("[a-f0-9]{8}(-[a-f0-9]{4}){4}[a-f0-9]{8}"));
    }

    @Test
    void testGenerateKeyPairWithCaseIdPart() {

        // Given
        final UUID caseId = UUID.randomUUID();
        final JWKPair keyPair = ReplyChannelKeyGenerator.generateKeyPair(caseId);

        // When
        final JWK publicKey = keyPair.getPublicKey();
        final JWK privateKey = keyPair.getPrivateKey();

        // Then
        assertThat(publicKey.getKeyID(), containsString("reply-key"));
        assertThat(publicKey.getKeyID().split(":")[1], is(caseId.toString()));

        assertThat(privateKey.getKeyID(), containsString("reply-key"));
        assertThat(privateKey.getKeyID().split(":")[1], is(caseId.toString()));
    }

    @Test
    void testPublicKeyValidity() throws JWKValidationException {

        // Given
        final JWKValidator jwkValidator = JWKValidator.withoutX5CValidation()
                .withErrorLogLevel(LogLevel.ERROR)
                .build();

        final JWKPair jwkPair = ReplyChannelKeyGenerator.generateKeyPair();

        // Then no exception is thrown
        jwkValidator.validate(jwkPair.getPublicKey().toRSAKey());
        jwkValidator.validate(jwkPair.getPrivateKey().toRSAKey());
    }
}